<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias"
    xmlns:isoold="http://www.ascc.net/xml/schematron"
    xmlns:iso="http://purl.oclc.org/dsdl/schematron"
    xmlns:xhtml="http://www.w3.org/1999/xhtml"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:fn="http://www.w3.org/2005/xpath-functions"
    xmlns:tns="http://www.ato.gov.au/schemas"
    xmlns:function="http://ato.gov.au/eCommerceFunctions"
    version="2.0"
    >


<xsl:function name="function:PathToNode" as="xs:string">
  <xsl:param name="node" as="node()?"/>

 <xsl:variable name="names" as="xs:string*">
   <xsl:for-each select="$node/ancestor-or-self::*">
     <xsl:variable name="ancestor" select="."/>
     <xsl:variable name="sibsOfSameName"
           select="$ancestor/../*[name() = name($ancestor)]"/>
     <xsl:sequence select="concat(name($ancestor),
         if (count($sibsOfSameName) &lt;= 1)
         then ''
         else concat(
        '[',function:IndexOfNode($sibsOfSameName,$ancestor),']'))"/>
   </xsl:for-each>
 </xsl:variable>
 <xsl:sequence select="string-join($names,'/')"/>

</xsl:function>

<xsl:function name="function:IndexOfNode" as="xs:integer*">
  <xsl:param name="nodes" as="node()*"/>
  <xsl:param name="nodeToFind" as="node()"/>

  <xsl:sequence select="
  for $seq in (1 to count($nodes))
  return $seq[$nodes[$seq] is $nodeToFind]
 "/>
</xsl:function>
<xsl:function name="function:substring-after-sequence" as="xs:string*">
  <xsl:param name="strings" as="xs:string*"/>
  <xsl:param name="afterString" as="xs:string"/>

  <xsl:sequence select="
     for $seq in $strings
     return substring-after($seq, $afterString)
    "/>

</xsl:function>


<xsl:function name="function:union" as="xs:anyAtomicType*"
              xmlns:functx="http://www.functx.com">
  <xsl:param name="arg1" as="xs:anyAtomicType*"/>
  <xsl:param name="arg2" as="xs:anyAtomicType*"/>

  <xsl:sequence select="
  distinct-values(($arg1, $arg2))
 "/>

</xsl:function>

<xsl:function name="function:NotSameValues" as="xs:boolean">
  <xsl:param name="nodes" as="xs:anyAtomicType*"/>

<xsl:value-of select="count(distinct-values($nodes)) = count($nodes)"  />

</xsl:function>

<xsl:function name="function:HasDuplicateValues" as="xs:boolean">
  <xsl:param name="nodes" as="xs:anyAtomicType*"/>

<xsl:value-of select="not(count(distinct-values($nodes)) = count($nodes))"  />

</xsl:function>


<xsl:function name="function:ConvertToDate" as="xs:date?">
    <xsl:param name="Day" as="xs:anyAtomicType?"/>
    <xsl:param name="Month" as="xs:anyAtomicType?"/>
    <xsl:param name="Year" as="xs:anyAtomicType?"/>
    <xsl:variable name="Date1" select="concat($Year,'-', substring-after($Month, '--'),'-', substring-after($Day, '---'))" as="xs:string?"/>
    <xsl:choose>
        <xsl:when test="$Date1 castable as xs:date">
            <xsl:variable name="castedDate1" select="xs:date($Date1)" as="xs:date"/>
            <xsl:sequence select="$castedDate1"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:sequence select= "current-date()"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:function>


<xsl:function name="function:Day" as="xs:double">
    <xsl:param name="Date1" as="xs:anyAtomicType?"/>
    <xsl:variable name="default" select="0" as="xs:double"/>
    <xsl:choose>
        <xsl:when test="$Date1 castable as xs:date ">
            <xsl:variable name="castedDate1" select="xs:date($Date1)" as="xs:date"/>
            <xsl:sequence select="day-from-date($castedDate1)"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:sequence select= "$default"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:function>


<xsl:function name="function:Month" as="xs:string">
    <xsl:param name="Date1" as="xs:anyAtomicType?"/>
    <xsl:variable name="default" select="'NotAMonth'" as="xs:string"/>
    <xsl:choose>
        <xsl:when test="$Date1 castable as xs:date ">
            <xsl:variable name="castedDate1" select="xs:date($Date1)" as="xs:date"/>
            <xsl:sequence select="format-date($castedDate1,'[MNn]')"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:sequence select= "$default"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:function>


<xsl:function name="function:MonthAsInt" as="xs:double">
    <xsl:param name="Date1" as="xs:anyAtomicType?"/>
    <xsl:variable name="default" select="0" as="xs:double"/>
    <xsl:choose>
        <xsl:when test="$Date1 castable as xs:date ">
            <xsl:variable name="castedDate1" select="xs:date($Date1)" as="xs:date"/>
            <xsl:sequence select="month-from-date($castedDate1)"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:sequence select= "$default"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:function>


<xsl:function name="function:Year" as="xs:integer">
    <xsl:param name="Date1" as="xs:anyAtomicType?"/>
    <xsl:variable name="default" select="0" as="xs:integer"/>
    <xsl:choose>
        <xsl:when test="$Date1 castable as xs:date ">
            <xsl:variable name="castedDate1" select="xs:date($Date1)" as="xs:date"/>
            <xsl:sequence select="year-from-date($castedDate1)"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:sequence select= "$default"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:function>


  <xsl:function name="function:FinancialYear" as="xs:integer">
      <xsl:param name="Date1" as="xs:anyAtomicType?"/>
    <xsl:choose>
      <xsl:when test="$Date1 castable as xs:date ">
        <xsl:sequence select= "function:DateToFinancialYear($Date1,7)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select= "0"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <xsl:function name="function:CurrentFinancialYear" as="xs:integer">
        <xsl:sequence select= "function:DateToFinancialYear(current-date(),7)"/>
  </xsl:function>

  <xsl:function name="function:DateToFinancialYear" as="xs:integer">
      <xsl:param name="Date1" as="xs:anyAtomicType?"/>
    <xsl:param name="financialYearStartMonth" as="xs:integer"/>
    <xsl:choose>
      <xsl:when test="$Date1 castable as xs:date ">
        <xsl:variable name="datecast" as="xs:date"
             select="$Date1 cast as xs:date"/>
        <xsl:variable name="lyearOfdate" select="year-from-date($datecast)" as="xs:integer"/>
        <xsl:variable name="lmonthOfdate" select="month-from-date($datecast)" as="xs:integer"/>
        <xsl:choose>
          <xsl:when test="$financialYearStartMonth &gt; $lmonthOfdate ">
            <xsl:sequence select="$lyearOfdate"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:sequence select= "$lyearOfdate+1"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select= "0"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>


  <xsl:function name="function:IsDate" as="xs:boolean">
      <xsl:param name="Date1" as="xs:anyAtomicType"/>
      <xsl:variable name="is-datecast" as="xs:boolean"
               select="$Date1 castable as xs:date"/>
      <xsl:sequence select= "$is-datecast"/>
  </xsl:function>

  <xsl:function name="function:IsDate1GreaterOrEqualToDate2" as="xs:boolean">
      <xsl:param name="Date1" as="xs:anyAtomicType"/>
      <xsl:param name="Date2" as="xs:anyAtomicType"/>
      <xsl:choose>
          <xsl:when test="function:IsDate($Date1) and function:IsDate($Date2)">
              <xsl:variable name="castedDate1" select="xs:date($Date1)" as="xs:date"/>
              <xsl:variable name="castedDate2" select="xs:date($Date2)" as="xs:date"/>
              <xsl:sequence select= "$castedDate1 >= $castedDate2"/>
          </xsl:when>
          <xsl:otherwise>
              <xsl:sequence select="false()"/>
          </xsl:otherwise>
      </xsl:choose>
  </xsl:function>

  <xsl:function name="function:IsDate1GreaterThanDate2" as="xs:boolean">
      <xsl:param name="Date1" as="xs:anyAtomicType"/>
      <xsl:param name="Date2" as="xs:anyAtomicType"/>
      <xsl:choose>
          <xsl:when test="function:IsDate($Date1) and function:IsDate($Date2)">
              <xsl:variable name="castedDate1" select="xs:date($Date1)" as="xs:date"/>
              <xsl:variable name="castedDate2" select="xs:date($Date2)" as="xs:date"/>
              <xsl:sequence select= "$castedDate1 > $castedDate2"/>
          </xsl:when>
          <xsl:otherwise>
              <xsl:sequence select="false()"/>
          </xsl:otherwise>
      </xsl:choose>
  </xsl:function>


<!-- ========================================================================
     function : FBTYear
     ========================================================================
     Description	
       This function get FBT year
     ========================================================================
    Input Parameters
         Input Parameters
         Date1 as Date or string: anyAtomicType
          <xsl:param name="Date1" as="xs:anyAtomicType?"/>
     ========================================================================
     Output : FBT Year
     If para pass in as a date -> Return FBTYear( If Month >3 then FBT Yr= Ry+1) else FBTYr=Year
     If Para pass in  not a Date -> Return FBT Year =0

     ======================================================================== -->


<xsl:function name="function:FBTYear" as="xs:integer">
    <xsl:param name="Date1" as="xs:anyAtomicType?"/>
    <xsl:variable name="fbtYear" select="0" as="xs:integer"/>

    <xsl:choose>
        <xsl:when test="$Date1 castable as xs:date ">
            <xsl:variable name="castedDate1" select="xs:date($Date1)" as="xs:date"/>

            <xsl:choose>
                <xsl:when test=" month-from-date($Date1) &gt; 3 ">
                   <xsl:sequence select="year-from-date($Date1)+1"/>
                </xsl:when>
                <xsl:otherwise>
                   <xsl:sequence select= "year-from-date($Date1)"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
            <xsl:sequence select= "$fbtYear"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:function>

<xsl:function name="function:NotMonetary" as="xs:boolean">
    <xsl:param name="field" as="xs:anyAtomicType?"/>
    <xsl:param name="sign" as="xs:string"/>
    <xsl:param name="digits" as="xs:integer"/>
    <xsl:param name="decimals" as="xs:integer"/>


    <xsl:variable name="fieldAsString" select="string($field)" as="xs:string"/>
    <xsl:variable name="signExpression" select="if ($sign = 'U') then '^' else '^-?'" />
    <xsl:variable name="decimalsExpression" select="if ($decimals > 0) then concat('(\.\d{1,',$decimals, '})?$') else '$'" />
    <xsl:variable name="digitsToUse" select="$digits - $decimals" as="xs:integer" />

      <xsl:value-of select= "string-length(normalize-space($fieldAsString)) > 0 and not($field castable as xs:decimal and matches($fieldAsString, concat($signExpression, '\d{1,', $digitsToUse,'}',$decimalsExpression)))"  /> 
</xsl:function>


<xsl:function name="function:NotNumeric" as="xs:boolean">
    <xsl:param name="field" as="xs:anyAtomicType?"/>
    <xsl:param name="sign" as="xs:string"/>
    <xsl:param name="digits" as="xs:integer"/>
    <xsl:param name="decimals" as="xs:integer"/>


    <xsl:variable name="fieldAsString" select="string($field)" as="xs:string"/>
    <xsl:variable name="signExpression" select="if ($sign = 'U') then '^' else '^-?'" />
    <xsl:variable name="decimalsExpression" select="if ($decimals > 0) then concat('(\.\d{1,',$decimals, '})?$') else '$'" />
    <xsl:variable name="digitsToUse" select="$digits - $decimals" as="xs:integer" />

      <xsl:value-of select= "string-length(normalize-space($fieldAsString)) > 0 and not($field castable as xs:decimal and matches($fieldAsString, concat($signExpression, '\d{1,', $digitsToUse,'}',$decimalsExpression)))"  /> 
</xsl:function>


<xsl:function name="function:OutsideRange" as="xs:boolean">
    <xsl:param name="field1" as="xs:anyAtomicType?"/>
    <xsl:param name="expressionResult" as="xs:anyAtomicType?"/>
    <xsl:param name="range" as="xs:integer"/>

    <xsl:variable name="field1AsDouble" select="number($field1)" as="xs:double"/>
    <xsl:variable name="expressionResultAsDouble" select="number($expressionResult)" as="xs:double"/>

    <xsl:value-of select= "($field1AsDouble &lt; ($expressionResultAsDouble - $range)) or ($field1AsDouble &gt; ($expressionResultAsDouble + $range))"  />
</xsl:function>


<xsl:function name="function:FailsUSIAlgorithm" as="xs:boolean">
    <xsl:param name="USI" as="xs:string?"/>
    <xsl:param name="ABN" as="xs:string?"/>
    <xsl:choose>
        <xsl:when test="string-length(normalize-space(($USI)[1])) &lt; 13 or string-length(normalize-space(($ABN)[1])) &lt; 11 ">
            <xsl:value-of select= "false()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:variable name="StartsWithABN" select="substring($USI,1,11) = $ABN" as="xs:boolean"/>
          <xsl:variable name="EndsWithNumeric" select="substring($USI,12,14) castable as xs:integer" as="xs:boolean"/>
          <xsl:variable name="StartsWithAlpha" select="matches(substring($USI,1,3), '[A-Z]*', 'i')" as="xs:boolean"/>
          <xsl:variable name="MiddleIsNumeric" select="substring($USI,4,7) castable as xs:integer" as="xs:boolean"/>
          <xsl:variable name="EndsWithAlpha" select="matches(substring($USI,8,9), '[A-Z]*', 'i')" as="xs:boolean"/>
          <xsl:choose>
              <xsl:when test="($StartsWithABN and $EndsWithNumeric) or ($StartsWithAlpha and $MiddleIsNumeric and $EndsWithAlpha) ">
                  <xsl:value-of select= "false()"/>
              </xsl:when>
              <xsl:otherwise>
                  <xsl:value-of  select= "true()"/>
              </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
       </xsl:choose>

</xsl:function>


<xsl:function name="function:FailsTANAlgorithm" as="xs:boolean">
    <xsl:param name="TAN_Initial" as="xs:string"/>
    <xsl:variable name="TANNoSpace" select="translate($TAN_Initial,' ','')" as="xs:string"/>
    <xsl:variable name="TANNumberString" select=" string(number($TANNoSpace))" as="xs:string"/>
    <xsl:variable name="TANlength" select="string-length($TANNoSpace)" as="xs:integer"/>
    <xsl:variable name="TANWSum1" select=" 7*number(substring($TANNoSpace,1,1))" as="xs:double"/>
    <xsl:variable name="TANWSum2" select="9*number(substring($TANNoSpace,2,1)) " as="xs:double"/>
    <xsl:variable name="TANWSum3" select="8*number(substring($TANNoSpace,3,1))" as="xs:double"/>
    <xsl:variable name="TANWSum4" select="4*number(substring($TANNoSpace,4,1))" as="xs:double"/>
    <xsl:variable name="TANWSum5" select="6*number(substring($TANNoSpace,5,1))" as="xs:double"/>
    <xsl:variable name="TANWSum6" select="3*number(substring($TANNoSpace,6,1))" as="xs:double"/>
    <xsl:variable name="TANWSum7" select="5*number(substring($TANNoSpace,7,1)) " as="xs:double"/>
    <xsl:variable name="TANWSum8" select="1*number(substring($TANNoSpace,8,1))" as="xs:double"/>
    <xsl:variable name="SumTANWSum" select="$TANWSum1 + $TANWSum2 + $TANWSum3 + $TANWSum4 + $TANWSum5 + $TANWSum6 + $TANWSum7 + $TANWSum8" as="xs:double"/>
    <xsl:choose>
        <xsl:when test="($TANlength = 8) and ($SumTANWSum mod 11 ) = 0">
            <xsl:value-of select= "false()"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:value-of  select= "true()"/>
        </xsl:otherwise>
    </xsl:choose>

</xsl:function>


<!--ABN weighingFactors = { 10, 1, 3, 5, 7, 9, 11, 13, 15, 17, 19 };-->
<xsl:function name="function:FailsABNAlgorithm" as="xs:boolean">
    <xsl:param name="ABN_Initial" as="xs:string"/>
    <xsl:variable name="ABNNoSpace" select="translate($ABN_Initial,' ','')" as="xs:string"/>
    <xsl:variable name="ABNNumbeString" select=" string(number($ABNNoSpace))" as="xs:string"/>
    <xsl:variable name="ABNlength" select="string-length($ABNNoSpace)" as="xs:double"/>
    <xsl:variable name="ABNWSum1" select="10*(number(substring($ABNNoSpace,1,1))-1) " as="xs:double"/>
    <xsl:variable name="ABNWSum2" select="1*number(substring($ABNNoSpace,2,1)) " as="xs:double"/>
    <xsl:variable name="ABNWSum3" select="3*number(substring($ABNNoSpace,3,1))" as="xs:double"/>
    <xsl:variable name="ABNWSum4" select="5*number(substring($ABNNoSpace,4,1))" as="xs:double"/>
    <xsl:variable name="ABNWSum5" select=" 7*number(substring($ABNNoSpace,5,1))" as="xs:double"/>
    <xsl:variable name="ABNWSum6" select=" 9*number(substring($ABNNoSpace,6,1))" as="xs:double"/>
    <xsl:variable name="ABNWSum7" select="11*number(substring($ABNNoSpace,7,1)) " as="xs:double"/>
    <xsl:variable name="ABNWSum8" select="13*number(substring($ABNNoSpace,8,1))" as="xs:double"/>
    <xsl:variable name="ABNWSum9" select=" 15*number(substring($ABNNoSpace,9,1))" as="xs:double"/>
    <xsl:variable name="ABNWSum10" select="17*number(substring($ABNNoSpace,10,1)) " as="xs:double"/>
    <xsl:variable name="ABNWSum11" select="19*number(substring($ABNNoSpace,11,1))" as="xs:double"/>
    <xsl:variable name="SumABNWSum" select="$ABNWSum1 + $ABNWSum2 + $ABNWSum3 + $ABNWSum4 + $ABNWSum5 + $ABNWSum6 + $ABNWSum7 + $ABNWSum8 + $ABNWSum9 + $ABNWSum10 + $ABNWSum11" as="xs:double"/>
    <xsl:choose>
        <xsl:when test=" ($ABNlength = 11 ) and ($SumABNWSum mod 89 ) = 0 ">
            <xsl:value-of select= "false()"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:value-of   select= "true()"/>
        </xsl:otherwise>
    </xsl:choose>

</xsl:function>


<!--int checkDigit = Int32.Parse(cleanACN[8].ToString());
    ACN weighingFactors = { 8, 7, 6, 5, 4, 3, 2, 1 };
    int remainder = weightedSum % 10;
    return checkDigit == (10 - remainder) % 10;-->
<xsl:function name="function:FailsACNAlgorithm" as="xs:boolean">
    <xsl:param name="ACN_Initial" as="xs:string"/>
    <xsl:variable name="ACNNoSpace" select="translate($ACN_Initial,' ','')" as="xs:string"/>
    <xsl:variable name="ACNNumberString" select=" string(number($ACNNoSpace))" as="xs:string"/>

    <xsl:variable name="ACNLength" select="string-length($ACNNoSpace)" as="xs:double"/>
    <xsl:variable name="ACNWSum1" select="8*number(substring($ACNNoSpace,1,1)) " as="xs:double"/>
    <xsl:variable name="ACNWSum2" select="7*number(substring($ACNNoSpace,2,1))" as="xs:double"/>
    <xsl:variable name="ACNWSum3" select="6*number(substring($ACNNoSpace,3,1))" as="xs:double"/>
    <xsl:variable name="ACNWSum4" select="5*number(substring($ACNNoSpace,4,1))" as="xs:double"/>
    <xsl:variable name="ACNWSum5" select=" 4*number(substring($ACNNoSpace,5,1))" as="xs:double"/>
    <xsl:variable name="ACNWSum6" select="3*number(substring($ACNNoSpace,6,1)) " as="xs:double"/>
    <xsl:variable name="ACNWSum7" select="2*number(substring($ACNNoSpace,7,1))" as="xs:double"/>
    <xsl:variable name="ACNWSum8" select="1*number(substring($ACNNoSpace,8,1))" as="xs:double"/>
    <xsl:variable name="ACNcheckDigit" select="number(substring($ACNNoSpace,9,1))" as="xs:double"/>
    <xsl:variable name="SumACNWSum" select=" $ACNWSum1 + $ACNWSum2 + $ACNWSum3 + $ACNWSum4 + $ACNWSum5 + $ACNWSum6 + $ACNWSum7 + $ACNWSum8 " as="xs:double"/>
    <xsl:variable name="ACNremainder" select="$SumACNWSum mod 10 " as="xs:double"/>
    <xsl:choose>
        <xsl:when test=" ($ACNLength = 9) and ((10-$ACNremainder ) mod 10) = $ACNcheckDigit ">
            <xsl:value-of select= "false()"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:value-of   select= "true()"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:function>


<!--TFN weighingFactors = { 1, 4, 3, 7, 5, 8, 6, 9, 10 };-->
<xsl:function name="function:FailsTFNAlgorithm" as="xs:boolean">
<xsl:param name="TFN_Initial" as="xs:anyAtomicType?"/>
<xsl:choose>
  <xsl:when test="string-length($TFN_Initial) > 0">
    <xsl:variable name="TFNNoSpace" select="translate($TFN_Initial,' ','')" as="xs:string"/>
    <xsl:variable name="TFNNumberString" select="fn:replace($TFNNoSpace, '^0+', '' )" as="xs:string"/>
    <xsl:variable name="TFNlength" select="string-length($TFNNumberString)" as="xs:double"/>
    <xsl:variable name="TFNWSum1" select="1*number(substring($TFNNumberString,1,1))" as="xs:double"/>
    <xsl:variable name="TFNWSum2" select="4*number(substring($TFNNumberString,2,1))" as="xs:double"/>
    <xsl:variable name="TFNWSum3" select="3*number(substring($TFNNumberString,3,1))" as="xs:double"/>
    <xsl:variable name="TFNWSum4" select="7*number(substring($TFNNumberString,4,1))" as="xs:double"/>
    <xsl:variable name="TFNWSum5" select="5*number(substring($TFNNumberString,5,1))" as="xs:double"/>
    <xsl:variable name="TFNWSum6" select="8*number(substring($TFNNumberString,6,1))" as="xs:double"/>
    <xsl:variable name="TFNWSum7" select="6*number(substring($TFNNumberString,7,1))" as="xs:double"/>
    <xsl:variable name="TFNWSum8" select="9*number(substring($TFNNumberString,8,1))" as="xs:double"/>
    <xsl:variable name="TFNWSum9" select="10*number(substring($TFNNumberString,9,1))" as="xs:double"/>
    <xsl:variable name="TFNLg8WSum8" select="0" as="xs:double"/>
    <xsl:variable name="TFNLg8WSum9" select="10*number(substring($TFNNumberString,8,1))" as="xs:double"/>
    <xsl:variable name="SumTFNLg8WSum" select="$TFNWSum1 + $TFNWSum2 + $TFNWSum3 + $TFNWSum4 + $TFNWSum5 + $TFNWSum6 + $TFNWSum7 + $TFNLg8WSum8 +$TFNLg8WSum9" as="xs:double"/>
    <xsl:variable name="SumTFNLg9WSum" select=" $TFNWSum1 + $TFNWSum2 + $TFNWSum3 + $TFNWSum4 + $TFNWSum5 + $TFNWSum6 + $TFNWSum7 + $TFNWSum8 +$TFNWSum9" as="xs:double"/>
    <xsl:choose>
      <xsl:when test=" ($TFNlength = 9 and ($SumTFNLg9WSum mod 11)=0 ) or ($TFNlength = 8 and ($SumTFNLg8WSum mod 11)=0 ) ">
        <xsl:value-of select= "false()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of   select= "true()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:when>
  <xsl:otherwise>
    <xsl:sequence select="true()"/>
  </xsl:otherwise>
</xsl:choose>
</xsl:function>

<xsl:function name="function:ConditionalValue" as="xs:double">
    <xsl:param name="BooleanExpression" as="xs:anyAtomicType?"/>
    <xsl:param name="TrueValue" as="xs:anyAtomicType?"/>
    <xsl:param name="FalseValue" as="xs:anyAtomicType?"/>
    <xsl:choose>
        <xsl:when test="$BooleanExpression">
            <xsl:value-of select="$TrueValue"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:value-of select= "$FalseValue"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:function>


<xsl:function name="function:AnyOccurrenceNull" as="xs:boolean">
    <xsl:param name="parentNodes" as="node()*"/>
    <xsl:param name="childNodes" as="node()*"/>
    <xsl:choose>
	    <xsl:when test="count($parentNodes) &gt; 0 and count($parentNodes) != count($childNodes)">
 		    <xsl:value-of select="true()"/>
	    </xsl:when>
	    <xsl:otherwise>
	 	    <xsl:value-of select="false()"/>
	    </xsl:otherwise>
    </xsl:choose>
</xsl:function>

<xsl:function name="function:AsNumeric" as="xs:integer">
    <xsl:param name="field" as="xs:string?"/>
    <xsl:variable name="default" select="0" as="xs:integer"/>
    <xsl:variable name="allowed-characters">0123456789</xsl:variable>
    <xsl:choose>
	     <xsl:when test="$field != '' and string-length(translate($field, $allowed-characters, '')) = 0">
 		    <xsl:value-of select="xs:integer($field)" />
	    </xsl:when>
	    <xsl:otherwise>
	 	    <xsl:value-of select= "$default" />
	    </xsl:otherwise>
    </xsl:choose>
</xsl:function>

  <xsl:function name="function:RegexMatch" as="xs:boolean">
    <xsl:param name="field" as="xs:anyAtomicType?"/>
    <xsl:param name="pattern" as="xs:anyAtomicType?"/>
    <xsl:param name="flags" as="xs:anyAtomicType?"/>

    <xsl:variable name="fieldAsString" select="string($field)" as="xs:string"/>
    <xsl:variable name="patternAsString" select="string($pattern)" as="xs:string"/>
    <xsl:variable name="flagsAsString" select="string($flags)" as="xs:string"/>

    <xsl:value-of select= "matches($fieldAsString,$patternAsString,$flagsAsString)"/>
  </xsl:function>


</xsl:stylesheet>
