using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Text.RegularExpressions;
using Au.Gov.Sbr.Xbrl.Document;
using DataContracts;
using Au.Gov.Sbr.Xbrl.Metadata;
using System.Runtime.CompilerServices;
using System.IO;
using System.Text;
using System.Xml.Schema;
using System.Reflection;
using System.Runtime.Serialization; 

namespace Ato.EN.IntegrationServices.CodeGenerationRDTIS {
    public partial class RDTIS2017Validator {
    
    #region Cross Form Rules - not supported yet

        #region VR.ATO.GEN.402009

        /*  VR.ATO.GEN.402009
                    Your supplied TFN does not match the TFN supplied on the form it was submitted with

                Legacy Rule Format:
                    IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                    RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^TFN <> ^ParentTFN

        Data Elements:
            
                    ^ParentTFN = ParentTFN
            
                    ^TFN = RP
                    */
        protected void VRATOGEN402009(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.402009
            
            assertion = false; // = ((report.RPExists == true) && (report.RPIdentifierTFN != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.402009", Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402009

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on the schedule does not match period start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCTFOROccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on the schedule does not match period start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPYPreviousOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on the schedule does not match period start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCTAUSOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on the schedule does not match period start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on the schedule does not match period start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(gmContext.OccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on the schedule does not match period end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP
                    */
        protected void VRATOGEN438001(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPExists == true) && (report.RPEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on the schedule does not match period end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.YPrevious
                    */
        protected void VRATOGEN438001_(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPYPreviousExists == true) && (report.RPYPreviousEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPYPreviousOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on the schedule does not match period end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.CTAUS
                    */
        protected void VRATOGEN438001__(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPCTAUSExists == true) && (report.RPCTAUSEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCTAUSOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on the schedule does not match period end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = GM
                    */
        protected void VRATOGEN438001___(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = (gmContext.EndDate != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(gmContext.OccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on the schedule does not match period end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.CTFOR
                    */
        protected void VRATOGEN438001____(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPCTFORExists == true) && (report.RPCTFOREndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCTFOROccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.RDTIS.438069

        /*  VR.ATO.RDTIS.438069
                    Refundable R&D tax offset is not equal to 43.5% of the Refundable tax offset Total notional R&D deductions

                Legacy Rule Format:
                    IF (PARENT RETURN:RP:period.startDate >= "2017-07-01") AND 
                    [RDTIS45] <> NULL AND
                    [RDTIS45] <= 100000000 AND
                    [RDTIS46] <> ([RDTIS45] * 0.435) +/- 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RDTIS45] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
                    [RDTIS46] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount

            Technical Business Rule Format:
                    (^ParentStartDate >=  AsDate('2017-07-01')) AND ^RDTIS45 <> NULL AND ^RDTIS45 <= 100000000 AND OutsideRange(^RDTIS46, ^RDTIS45 * 0.435, 1)

        Data Elements:
            
                    RP:^RDTIS46 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
            
                    RP:^RDTIS45 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
            
                    RP:^ParentStartDate = ParentStartDate
                    */
        protected void VRATORDTIS438069(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.RDTIS.438069
            
            assertion = false; // = ( >= AsDate(@"2017-07-01") && report.RDTIS45 != null && report.RDTIS45.GetValueOrDefault() <= 100000000 && OutsideRange(report.RDTIS46.GetValueOrDefault(), report.RDTIS45.GetValueOrDefault() * 0.435M, 1));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438073", Severity = ProcessMessageSeverity.Error,
                    Description = @"Refundable R&D tax offset is incorrect",
                    LongDescription = @"Refundable R&D tax offset is not equal to 43.5% of the Refundable tax offset Total notional R&D deductions",
                    //Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438069"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS45", Value = report.RDTIS45.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS46", Value = report.RDTIS46.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RDTIS.438069

        #region VR.ATO.RDTIS.438070

        /*  VR.ATO.RDTIS.438070
                    Refundable R&D tax offset is not equal to $43.5 million plus 30% of the amount of Refundable tax offset Total notional R&D deductions above $100 million

                Legacy Rule Format:
                    IF (PARENT RETURN:RP:period.startDate >= "2017-07-01") AND
                    [RDTIS45] <> NULL AND
                    [RDTIS45] > 100000000 AND
                    [RDTIS46] <> (43500000 + (([RDTIS45] – 100000000)* 0.30)) +/- 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RDTIS45] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
                    [RDTIS46] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount

            Technical Business Rule Format:
                    (^ParentStartDate >=  AsDate('2017-07-01')) AND ^RDTIS45 <> NULL AND ^RDTIS45 > 100000000 AND ^RDTIS46 <> (43500000 + ((^RDTIS45 - 100000000)* 0.30)) +/- 1

        Data Elements:
            
                    RP:^RDTIS46 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
            
                    RP:^RDTIS45 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
            
                    RP:^ParentStartDate = ParentStartDate
                    */
        protected void VRATORDTIS438070(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.RDTIS.438070
            
            assertion = false; // = ;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438074", Severity = ProcessMessageSeverity.Error,
                    Description = @"Refundable R&D tax offset is incorrect",
                    LongDescription = @"Refundable R&D tax offset is not equal to $43.5 million plus 30% of the amount of Refundable tax offset Total notional R&D deductions above $100 million",
                    //Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438070"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RDTIS.438070

        #region VR.ATO.RDTIS.438071

        /*  VR.ATO.RDTIS.438071
                    Non-refundable R&D tax offset is not equal to 38.5% of the Non-refundable tax offset Total notional R&D deductions

                Legacy Rule Format:
                    IF (PARENT RETURN:RP:period.startDate >= "2017-07-01") AND
                    [RDTIS47] <> NULL AND
                    [RDTIS47] <= 100000000 AND
                    [RDTIS48] <> ([RDTIS47] * 0.385) +/-1
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RDTIS47] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
                    [RDTIS48] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount

            Technical Business Rule Format:
                    (^ParentStartDate >=  AsDate('2017-07-01')) AND ^RDTIS47 <> NULL AND ^RDTIS47 <= 100000000 AND OutsideRange(^RDTIS48, ^RDTIS47 * 0.385, 1)

        Data Elements:
            
                    RP:^RDTIS48 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
            
                    RP:^RDTIS47 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
            
                    RP:^ParentStartDate = ParentStartDate
                    */
        protected void VRATORDTIS438071(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.RDTIS.438071
            
            assertion = false; // = ( >= AsDate(@"2017-07-01") && report.RDTIS47 != null && report.RDTIS47.GetValueOrDefault() <= 100000000 && OutsideRange(report.RDTIS48.GetValueOrDefault(), report.RDTIS47.GetValueOrDefault() * 0.385M, 1));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438075", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-refundable R&D tax offset is incorrect",
                    LongDescription = @"Non-refundable R&D tax offset is not equal to 38.5% of the Non-refundable tax offset Total notional R&D deductions",
                    //Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438071"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS47", Value = report.RDTIS47.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS48", Value = report.RDTIS48.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RDTIS.438071

        #region VR.ATO.RDTIS.438072

        /*  VR.ATO.RDTIS.438072
                    Non-refundable R&D tax offset is not equal to $38.5 million plus 30% of the amount of Non-refundable tax offset Total notional R&D deductions
                    above $100 million

                Legacy Rule Format:
                    IF (PARENT RETURN:RP:period.startDate >= "2017-07-01") AND 
                    [RDTIS47] <> NULL AND
                    [RDTIS47] > 100000000 AND
                    [RDTIS48] <> (38500000 + (([RDTIS47] – 100000000) * 0.30)) +/- 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RDTIS47] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
                    [RDTIS48] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount

            Technical Business Rule Format:
                    (^ParentStartDate >=  AsDate('2017-07-01')) AND ^RDTIS47 <> NULL AND ^RDTIS47 > 100000000 AND ^RDTIS48 <> (38500000 + ((^RDTIS47 - 100000000) * 0.30)) +/- 1

        Data Elements:
            
                    RP:^RDTIS48 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
            
                    RP:^RDTIS47 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
            
                    RP:^ParentStartDate = ParentStartDate
                    */
        protected void VRATORDTIS438072(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.RDTIS.438072
            
            assertion = false; // = ;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438076", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-refundable R&D tax offset is incorrect",
                    LongDescription = @"Non-refundable R&D tax offset is not equal to $38.5 million plus 30% of the amount of Non-refundable tax offset Total notional R&D deductions
above $100 million",
                    //Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438072"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RDTIS.438072

        #region VR.ATO.RDTIS.438077

        /*  VR.ATO.RDTIS.438077
                    Refundable R&D tax offset is not equal to 45% of the Refundable tax offset Total notional R&D deductions

                Legacy Rule Format:
                    IF (PARENT RETURN:RP:period.startDate < "2017-07-01") AND
                    [RDTIS45] <> NULL AND
                    [RDTIS45] <= 100000000 AND
                    [RDTIS46] <> ([RDTIS45] * 0.45) +/- 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [RDTIS45] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
                    [RDTIS46] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount

            Technical Business Rule Format:
                    (^ParentStartDate <  AsDate('2017-07-01')) AND ^RDTIS45 <> NULL AND ^RDTIS45 <= 100000000 AND OutsideRange(^RDTIS46, ^RDTIS45 * 0.45, 1)

        Data Elements:
            
                    RP:^RDTIS46 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
            
                    RP:^RDTIS45 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
            
                    RP:^ParentStartDate = ParentStartDate
                    */
        protected void VRATORDTIS438077(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.RDTIS.438077
            
            assertion = false; // = ( < AsDate(@"2017-07-01") && report.RDTIS45 != null && report.RDTIS45.GetValueOrDefault() <= 100000000 && OutsideRange(report.RDTIS46.GetValueOrDefault(), report.RDTIS45.GetValueOrDefault() * 0.45M, 1));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438077", Severity = ProcessMessageSeverity.Error,
                    Description = @"Refundable R&D tax offset is incorrect",
                    LongDescription = @"Refundable R&D tax offset must equal 45% of the Refundable tax offset Total notional R&D deductions.",
                    //Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438077"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS45", Value = report.RDTIS45.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS46", Value = report.RDTIS46.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RDTIS.438077

        #region VR.ATO.RDTIS.438078

        /*  VR.ATO.RDTIS.438078
                    Refundable R&D tax offset is not equal to $45 million plus 30% of the amount of Refundable tax offset Total notional R&D deductions above $100 million

                Legacy Rule Format:
                    IF (PARENT RETURN:RP:period.startDate < "2017-07-01") AND
                    [RDTIS45] <> NULL AND
                    [RDTIS45] > 100000000 AND
                    [RDTIS46] <> (45000000 + (([RDTIS45] – 100000000)* 0.30)) +/- 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [RDTIS45] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
                    [RDTIS46] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount

            Technical Business Rule Format:
                    (^ParentStartDate <  AsDate('2017-07-01')) AND ^RDTIS45 <> NULL AND ^RDTIS45 > 100000000 AND ^RDTIS46 <> (45000000 + ((^RDTIS45 - 100000000)* 0.30)) +/- 1

        Data Elements:
            
                    RP:^RDTIS46 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
            
                    RP:^RDTIS45 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
            
                    RP:^ParentStartDate = ParentStartDate
                    */
        protected void VRATORDTIS438078(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.RDTIS.438078
            
            assertion = false; // = ;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438078", Severity = ProcessMessageSeverity.Error,
                    Description = @"Refundable R&D tax offset is incorrect",
                    LongDescription = @"Refundable R&D tax offset must be equal to $45 million plus 30% of the amount of Refundable tax offset Total notional R&D deductions above $100 million.",
                    //Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438078"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RDTIS.438078

        #region VR.ATO.RDTIS.438079

        /*  VR.ATO.RDTIS.438079
                    Non-refundable R&D tax offset is not equal to 40% of the Non-refundable tax offset Total notional R&D deductions

                Legacy Rule Format:
                    IF (PARENT RETURN:RP:period.startDate < "2017-07-01") AND
                    [RDTIS47] <> NULL AND
                    [RDTIS47] <= 100000000 AND
                    [RDTIS48] <> ([RDTIS47] * 0.4) +/-1
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RDTIS47] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
                    [RDTIS48] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount

            Technical Business Rule Format:
                    (^ParentStartDate <  AsDate('2017-07-01')) AND ^RDTIS47 <> NULL AND ^RDTIS47 <= 100000000 AND OutsideRange(^RDTIS48, ^RDTIS47 * 0.4, 1)

        Data Elements:
            
                    RP:^RDTIS48 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
            
                    RP:^RDTIS47 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
            
                    RP:^ParentStartDate = ParentStartDate
                    */
        protected void VRATORDTIS438079(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.RDTIS.438079
            
            assertion = false; // = ( < AsDate(@"2017-07-01") && report.RDTIS47 != null && report.RDTIS47.GetValueOrDefault() <= 100000000 && OutsideRange(report.RDTIS48.GetValueOrDefault(), report.RDTIS47.GetValueOrDefault() * 0.4M, 1));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438079", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-refundable R&D tax offset is incorrect",
                    LongDescription = @"Non-refundable R&D tax offset must equal 40% of the Non-refundable tax offset Total notional R&D deductions.",
                    //Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438079"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS47", Value = report.RDTIS47.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "RDTIS48", Value = report.RDTIS48.GetValueOrDefault().ToString() });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RDTIS.438079

        #region VR.ATO.RDTIS.438080

        /*  VR.ATO.RDTIS.438080
                    Non-refundable R&D tax offset is not equal to $40 million plus 30% of the amount of Non-refundable tax offset Total notional R&D deductions
                    above $100 million

                Legacy Rule Format:
                    IF (PARENT RETURN:RP:period.startDate < "2017-07-01") AND
                    [RDTIS47] <> NULL AND
                    [RDTIS47] > 100000000 AND
                    [RDTIS48] <> (40000000 + (([RDTIS47] – 100000000) * 0.30)) +/- 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RDTIS47] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
                    [RDTIS48] = RDTIS:RP:gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount

            Technical Business Rule Format:
                    (^ParentStartDate <  AsDate('2017-07-01')) AND ^RDTIS47 <> NULL AND ^RDTIS47 > 100000000 AND ^RDTIS48 <> (40000000 + ((^RDTIS47 - 100000000) * 0.30)) +/- 1

        Data Elements:
            
                    RP:^RDTIS48 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
            
                    RP:^RDTIS47 = gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
            
                    RP:^ParentStartDate = ParentStartDate
                    */
        protected void VRATORDTIS438080(List<ProcessMessageDocument> response, RDTIS2013 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.RDTIS.438080
            
            assertion = false; // = ;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438080", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-refundable R&D tax offset is incorrect",
                    LongDescription = @"Non-refundable R&D tax offset must be equal to $40 million plus 30% of the amount of Non-refundable tax offset Total notional R&D deductions above $100 million.",
                    //Location = "/xbrli:xbrl/gfagc.02.04:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438080"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.RDTIS.438080
    #endregion Cross Form Rules - not supported yet
        #region ValidateCrossFormRules

        public List<ProcessMessageDocument> ValidateCrossFormRules(RDTIS2013 report)
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            #region Cross Form Rules
            VRATOGEN402009(response, report);
            VRATOGEN438000(response, report);
            VRATOGEN438000_(response, report);
            VRATOGEN438000__(response, report);
            VRATOGEN438000___(response, report);
            VRATOGEN438000____(response, report);
            VRATOGEN438001(response, report);
            VRATOGEN438001_(response, report);
            VRATOGEN438001__(response, report);
            VRATOGEN438001___(response, report);
            VRATOGEN438001____(response, report);
            VRATORDTIS438069(response, report);
            VRATORDTIS438070(response, report);
            VRATORDTIS438071(response, report);
            VRATORDTIS438072(response, report);
            VRATORDTIS438077(response, report);
            VRATORDTIS438078(response, report);
            VRATORDTIS438079(response, report);
            VRATORDTIS438080(response, report);
            #endregion Cross Form Rules
            return response;
        }

        #endregion
    }
}
