using System;
using System.Collections.Generic;
using Ato.EN.IntegrationServices.Document.DataContracts;
namespace Ato.EN.IntegrationServices.CodeGenerationSMSFAR
{
    public partial class SMSFAR2017Validator
    {

        #region Cross Form Rules - not supported yet

        #region VR.ATO.GEN.438022

        /*  VR.ATO.GEN.438022
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business active asset reduction (Subdivision 152-C)", then Small business active asset reduction amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "A - Small business active asset reduction") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS127] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS127] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount

            Technical Business Rule Format:
                    (^SMSFAR328 = 'A - Small business active asset reduction') AND (CountDocument('CGTS') = 1) AND ^CGTS127 = NULL

        Data Elements:
            
                    RP:^SMSFAR328 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS127 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount
                    */
        protected void VRATOGEN438022(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438022

            assertion = false; // = (SMSFAR328 == @"A - Small business active asset reduction" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438022",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business active asset reduction amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business active asset reduction (Subdivision 152-C)"", then Small business active asset reduction amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438022" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR328", Value = SMSFAR328 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438022

        #region VR.ATO.GEN.438023

        /*  VR.ATO.GEN.438023
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business retirement exemption (Subdivision152-D)", then Small business retirement exemption amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "B - Small business retirement exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS128] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS128] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount

            Technical Business Rule Format:
                    (^SMSFAR328 = 'B - Small business retirement exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS128 = NULL

        Data Elements:
            
                    RP:^SMSFAR328 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS128 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount
                    */
        protected void VRATOGEN438023(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438023

            assertion = false; // = (SMSFAR328 == @"B - Small business retirement exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business retirement exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business retirement exemption (Subdivision152-D)"", then Small business retirement exemption amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438023" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR328", Value = SMSFAR328 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438023

        #region VR.ATO.GEN.438024

        /*  VR.ATO.GEN.438024
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business roll-over (Subdivision 152-E)", then Small business roll-over amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "C - Small business roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS129] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS129] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount

            Technical Business Rule Format:
                    (^SMSFAR328 = 'C - Small business roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS129 = NULL

        Data Elements:
            
                    RP:^SMSFAR328 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS129 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount
                    */
        protected void VRATOGEN438024(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438024

            assertion = false; // = (SMSFAR328 == @"C - Small business roll-over" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438024",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business roll-over (Subdivision 152-E)"", then Small business roll-over amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438024" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR328", Value = SMSFAR328 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438024

        #region VR.ATO.GEN.438025

        /*  VR.ATO.GEN.438025
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business 15 year exemption (Subdivision152-B)", then Small business 15 year exemption amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "D - Small business 15 year exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS138] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS138] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount

            Technical Business Rule Format:
                    (^SMSFAR328 = 'D - Small business 15 year exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS138 = NULL

        Data Elements:
            
                    RP:^SMSFAR328 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS138 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount
                    */
        protected void VRATOGEN438025(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438025

            assertion = false; // = (SMSFAR328 == @"D - Small business 15 year exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438025",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business 15 year exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business 15 year exemption (Subdivision152-B)"", then Small business 15 year exemption amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438025" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR328", Value = SMSFAR328 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438025

        #region VR.ATO.GEN.438026

        /*  VR.ATO.GEN.438026
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Foreign resident CGT exemption (Division 855)", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "E - Foreign resident CGT exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS140] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS140] = CGTS:RP:rvctc3.02.11:Capital.Gains.ForeignResidentExemption.Amount

            Technical Business Rule Format:
                    (^SMSFAR328 = 'E - Foreign resident CGT exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS140 = NULL

        Data Elements:
            
                    RP:^SMSFAR328 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS140 = CGTS:RP:rvctc3.02.11:Capital.Gains.ForeignResidentExemption.Amount
                    */
        protected void VRATOGEN438026(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438026

            assertion = false; // = (SMSFAR328 == @"E - Foreign resident CGT exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438026",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a foreign resident amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Foreign resident CGT exemption (Division 855)"", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438026" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR328", Value = SMSFAR328 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438026

        #region VR.ATO.GEN.438027

        /*  VR.ATO.GEN.438027
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Scrip for scrip roll-over (Subdivision 124-M)", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "F - Scrip for scrip roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS45] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS45] = CGTS:RP:rvctc3.02.11:Capital.Gains.ScriptForScriptRolloverDeferral.Amount

            Technical Business Rule Format:
                    (^SMSFAR328 = 'F - Scrip for scrip roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS45 = NULL

        Data Elements:
            
                    RP:^SMSFAR328 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS45 = CGTS:RP:rvctc3.02.11:Capital.Gains.ScriptForScriptRolloverDeferral.Amount
                    */
        protected void VRATOGEN438027(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438027

            assertion = false; // = (SMSFAR328 == @"F - Scrip for scrip roll-over" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438027",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded as a result of a scrip for scrip roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Scrip for scrip roll-over (Subdivision 124-M)"", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438027" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR328", Value = SMSFAR328 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438027

        #region VR.ATO.SMSFAR.436010

        /*  VR.ATO.SMSFAR.436010
                    The Family Trust TFN on the IEE must not be the same as the main entity TFN

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "IEE") > 0) AND RP:entity.identifier.TFN = (ANY OCCURRENCE OF([IEE50])) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE50] = IEE:RP:pyid.xx.xx:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0002.lodge.req.xx.xx:FamilyTrust)

            Technical Business Rule Format:
                    (CountDocument('IEE') > 0) AND AnyOccurrence(^IEE50, ^IEE50 = ^TFN)

        Data Elements:
            
                    ^IEE50 = IEE:RP:pyid.66.66:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0002.lodge.req.66.66:FamilyTrust)
            
                    ^TFN = RP
                    */
        protected void VRATOSMSFAR436010(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436010

            assertion = false; // = (@"IEE") > 0 && ( == null ? false : .Any(IEE50Repeat =>  == report.RPIdentifierTFN)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Family Trust TFN on the IEE must not be the same as the main entity TFN",
                    LongDescription = @"An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the tax file number of the entity making the election",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436010" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 

                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436010

        #region VR.ATO.SMSFAR.436038

        /*  VR.ATO.SMSFAR.436038
                    If Net capital gain is more than $10,000, a CGT schedule must be present

                Legacy Rule Format:
                    IF [SMSFAR33] > 10000 AND  (COUNT(SCHEDULE = "CGTS") = 0)  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [SMSFAR33] = SMSFAR:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    ^SMSFAR33 > 10000 AND  (CountDocument('CGTS') = 0)

        Data Elements:
            
                    RP:^SMSFAR33 = tns:Income.CapitalGainsNet.Amount
                    */
        protected void VRATOSMSFAR436038(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436038

            assertion = false; // = (report.SMSFAR33.GetValueOrDefault() > 10000 && @"CGTS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434034",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If Net capital gain is more than $10,000, a CGT schedule must be present",
                    //Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436038" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR33", Value = report.SMSFAR33.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436038

        #region VR.ATO.SMSFAR.436051

        /*  VR.ATO.SMSFAR.436051
                    Payment summary schedule must be present when Payments where ABN not quoted amount is present

                Legacy Rule Format:
                    IF [SMSFAR42] > 0 AND  AND (COUNT(SCHEDULE = "PSS") = 0) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [SMSFAR42] = SMSFAR:RP:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    ^SMSFAR42 > 0 AND (CountDocument('PSS') = 0)

        Data Elements:
            
                    RP:^SMSFAR42 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
                    */
        protected void VRATOSMSFAR436051(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436051

            assertion = false; // = (report.SMSFAR42.GetValueOrDefault() > 0 && @"PSS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434044",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment summary schedule must be present when Payments where ABN not quoted amount is present",
                    //Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436051" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR42", Value = report.SMSFAR42.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436051

        #region VR.ATO.SMSFAR.436052

        /*  VR.ATO.SMSFAR.436052
                    Gross payment amount must equal the sum of amounts in the Payment Summary schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM(ALL OCCURRENCES OF([PSS19]))) <> [SMSFAR42] 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS19] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [SMSFAR42] = SMSFAR:RP:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') = 1) AND (Sum(^PSS19)) <> ^SMSFAR42

        Data Elements:
            
                    RP:^SMSFAR42 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP:^PSS19 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    */
        protected void VRATOSMSFAR436052(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436052

            assertion = false; // = (@"PSS") == 1 && ( == null ? 0 : .Sum(f => f.)) != report.SMSFAR42.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434046",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payment amount must equal the sum of amounts in the Payment Summary schedule",
                    //Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436052" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR42", Value = report.SMSFAR42.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436052

        #region VR.ATO.SMSFAR.436053

        /*  VR.ATO.SMSFAR.436053
                    The 'Credit for tax withheld - where TFN or ABN not quoted' amount must equal the sum of the Tax withheld amounts on the Payment Summary schedule

                Legacy Rule Format:
                    IF COUNT(SCHEDULE = "PSS") = 1 AND [SMSFAR42] > 0 AND [SMSFAR80] + 1 < SUM(ALL OCCURRENCES OF([PSS23]))
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS23] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                    [SMSFAR42] = SMSFAR:RP:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [SMSFAR80] = SMSFAR:RP:rvctc2.02.02:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldTFNNotQuotedAndABNNotQuoted.Amount

            Technical Business Rule Format:
                    CountDocument('PSS') = 1 AND ^SMSFAR42 > 0 AND ^SMSFAR80 + 1 < Sum(^PSS23)

        Data Elements:
            
                    RP:^SMSFAR42 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP:^PSS23 = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
            
                    RP:^SMSFAR80 = tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldTFNNotQuotedAndABNNotQuoted.Amount
                    */
        protected void VRATOSMSFAR436053(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436053

            assertion = false; // = (@"PSS") == 1 && report.SMSFAR42.GetValueOrDefault() > 0 && report.SMSFAR80.GetValueOrDefault() + 1 < ( == null ? 0 : .Sum(f => f.)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.SMSFAR.436334",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax withheld amount must not be less than the sum of amounts present in the schedule",
                    LongDescription = @"The 'Credit for tax withheld - where TFN or ABN not quoted' amount must equal the sum of the Tax withheld amounts on the Payment Summary schedule",
                    //Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436053" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR42", Value = report.SMSFAR42.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR80", Value = report.SMSFAR80.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436053

        #region VR.ATO.SMSFAR.436072

        /*  VR.ATO.SMSFAR.436072
                    The 'Other income' amount must not be less than the sum of all 'Gross payment' amounts on the Payment Summary schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM(ALL OCCURRENCES OF([PSS18])) > [SMSFAR53])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS18] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    [SMSFAR53] = SMSFAR:RP:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') = 1) AND (Sum(^PSS18) > ^SMSFAR53)

        Data Elements:
            
                    RP:^SMSFAR53 = tns:Income.Other.Amount
            
                    RP:^PSS18 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    */
        protected void VRATOSMSFAR436072(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436072

            assertion = false; // = (@"PSS") == 1 && ( == null ? 0 : .Sum(f => f.)) > report.SMSFAR53.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434056",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other income must not be less than Gross payment amounts on Payment Summary schedule",
                    LongDescription = @"The 'Other income' amount must not be less than the sum of all 'Gross payment' amounts on the Payment Summary schedule",
                    //Location = "/xbrli:xbrl/tns:Income.Other.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436072" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR53", Value = report.SMSFAR53.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436072

        #region VR.ATO.SMSFAR.436118

        /*  VR.ATO.SMSFAR.436118
                    The 'Credit for tax withheld - foreign resident withholding (excluding capital gains)' amount on the main form must not be less than the sum of all 'Foreign resident withholding (excluding capital gains)' amounts on the Payment Summary schedule.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM(ALL OCCURRENCES OF([PSS22]))) > ([SMSFAR79] + 1)  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS22] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
                    [SMSFAR79] = SMSFAR:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') = 1) AND (Sum(^PSS22)) > (^SMSFAR79 + 1)

        Data Elements:
            
                    RP:^SMSFAR79 = tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            
                    RP:^PSS22 = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
                    */
        protected void VRATOSMSFAR436118(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436118

            assertion = false; // = (@"PSS") == 1 && ( == null ? 0 : .Sum(f => f.)) > report.SMSFAR79.GetValueOrDefault() + 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.SMSFAR.437137",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit for tax withheld - foreign resident withholding amount doesn't match the total amount on the schedule.",
                    LongDescription = @"The 'Credit for tax withheld - foreign resident withholding (excluding capital gains)' amount on the main form must not be less than the sum of all 'Foreign resident withholding (excluding capital gains)' amounts on the Payment Summary schedule.",
                    //Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436118" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR79", Value = report.SMSFAR79.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436118

        #region VR.ATO.SMSFAR.436127

        /*  VR.ATO.SMSFAR.436127
                    If the sum of 'Tax losses carried forward to later income years' and 'Net capital losses carried forward to later income years' is greater than $100000, a Losses schedule must be present

                Legacy Rule Format:
                    IF ([SMSFAR85] + [SMSFAR380]) > 100000 AND (COUNT(SCHEDULE = "LS") = 0) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [SMSFAR85] = SMSFAR:RP.Closing:rvctc3.02.02:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
                    [SMSFAR380] = SMSFAR:RP.Y0Plus.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (^SMSFAR85 + ^SMSFAR380) > 100000 AND (CountDocument('LS') = 0)

        Data Elements:
            
                    RP.Closing:^SMSFAR85 = tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
            
                    RP.Y0Plus.Closing(Instant):^SMSFAR380 = tns:Capital.Losses.CarriedForward.Net.Amount
                    */
        protected void VRATOSMSFAR436127(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436127

            assertion = false; // = (report.SMSFAR85.GetValueOrDefault() + report.SMSFAR380.GetValueOrDefault() > 100000 && @"LS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434085",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Losses schedule must be attached if losses carried forward exceeds $100,000",
                    LongDescription = @"If the sum of 'Tax losses carried forward to later income years' and 'Net capital losses carried forward to later income years' is greater than $100000, a Losses schedule must be present",
                    //Location = "/xbrli:xbrl/tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436127" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR85", Value = report.SMSFAR85.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR380", Value = report.SMSFAR380.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436127

        #region VR.ATO.SMSFAR.436129

        /*  VR.ATO.SMSFAR.436129
                    The total of the 'Tax losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Tax losses carried forward to later income years' on the main form

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "LS") = 1) AND [LS50] <> [SMSFAR85]  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [LS50] = LS:RP.ALL:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
                    [SMSFAR85] = SMSFAR:RP.Closing:rvctc3.02.02:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount

            Technical Business Rule Format:
                    (CountDocument('LS') = 1) AND ^LS50 <> ^SMSFAR85

        Data Elements:
            
                    RP.Closing:^SMSFAR85 = tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
            
                    RP.Closing:^LS50 = LS:RP.ALL:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
                    */
        protected void VRATOSMSFAR436129(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436129

            assertion = false; // = (@"LS") == 1 &&  != report.SMSFAR85.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax losses carried forward to later income years in schedule must equal amount in main form",
                    LongDescription = @"The total of the 'Tax losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Tax losses carried forward to later income years' on the main form",
                    //Location = "/xbrli:xbrl/tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436129" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR85", Value = report.SMSFAR85.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436129

        #region VR.ATO.SMSFAR.436194

        /*  VR.ATO.SMSFAR.436194
                    Family Trust Election status code selected requires a Family trust election, revocation or variation schedule to be completed and attached to return and there is no schedule present.

                Legacy Rule Format:
                    IF [SMSFAR163] = SET("R","V") AND  (COUNT(SCHEDULE = "FTER") = 0)
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [SMSFAR163] = SMSFAR:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
                    InSet(^SMSFAR163, '"R","V"') AND  (CountDocument('FTER') = 0)

        Data Elements:
            
                    RP:^SMSFAR163 = tns:Elections.FamilyTrustElectionRevocation.Code
                    */
        protected void VRATOSMSFAR436194(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436194

            assertion = false; // = (IsMatch(report.SMSFAR163, @"^(R|V)$") && @"FTER") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family trust election or revocation schedule must be present",
                    //Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436194" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR163", Value = report.SMSFAR163 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436194

        #region VR.ATO.SMSFAR.436196

        /*  VR.ATO.SMSFAR.436196
                    If an IEE schedule is present, the Interposed entity election status - Specified income year must be present

                Legacy Rule Format:
                    IF [SMSFAR164] = NULL and COUNT(SCHEDULE = "IEE") > 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [SMSFAR164] = SMSFAR:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year

            Technical Business Rule Format:
                    ^SMSFAR164 = NULL and CountDocument('IEE') > 0

        Data Elements:
            
                    RP:^SMSFAR164 = tns:Elections.InterposedEntityElectionStatus.Year
                    */
        protected void VRATOSMSFAR436196(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436196

            assertion = false; // = (report.SMSFAR164 == null && @"IEE") > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430178",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"IEE specified income year must be present",
                    LongDescription = @"If an IEE schedule is present, the Interposed entity election status - Specified income year must be present",
                    //Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436196" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR164", Value = report.SMSFAR164.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436196

        #region VR.ATO.SMSFAR.436197

        /*  VR.ATO.SMSFAR.436197
                    The Interposed Entity Election code indicates a revocation is being made, however no schedule is attached.

                Legacy Rule Format:
                    IF ([SMSFAR165] = "R") AND (COUNT(SCHEDULE = "IEE") = 0)
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [SMSFAR165] = SMSFAR:RP:rvctc3.02.00:Elections.InterposedEntityElectionRevocation.Code

            Technical Business Rule Format:
                    (^SMSFAR165 = 'R') AND (CountDocument('IEE') = 0)

        Data Elements:
            
                    RP:^SMSFAR165 = tns:Elections.InterposedEntityElectionRevocation.Code
                    */
        protected void VRATOSMSFAR436197(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436197

            assertion = false; // = (report.SMSFAR165 == @"R" && @"IEE") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430180",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If the Interposed Entity Election revocation code is present, an IEE schedule must be present",
                    //Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436197" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR165", Value = report.SMSFAR165 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436197

        #region VR.ATO.SMSFAR.436252

        /*  VR.ATO.SMSFAR.436252
                    The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.

                Legacy Rule Format:
                    IF (SMSFAR:RP:pyin.xx.xx:Report.TargetFinancial.Year <> NULL) AND (COUNT(SCHEDULE = "IEE") > 0) AND (ANY OCCURRENCE OF ([IEE116] = "R") AND ([IEE82] <> SMSFAR:RP:pyin.xx.xx:Report.TargetFinancial.Year))
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE82] = IEE:RP:rvctc3.xx.xx:Elections.Revocation.Year
                    [IEE116] = IEE:RP:rvctc3.xx.xx:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
                    (^SMSFAR1 <> NULL) AND (CountDocument('IEE') > 0) AND ((AnyOccurrence(^IEE116, ^IEE116 = 'R')) AND (^IEE82 <> ^SMSFAR1))

        Data Elements:
            
                    RP:^SMSFAR1 = tns:Report.TargetFinancial.Year
            
                    RP:^IEE82 = IEE:RP:rvctc3.66.66:Elections.Revocation.Year
            
                    RP:^IEE116 = IEE:RP:rvctc3.66.66:Elections.InterposedEntityElectionOrRevocation.Code
                    */
        protected void VRATOSMSFAR436252(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436252

            assertion = false; // = (report.SMSFAR1 != null && @"IEE") > 0 && (( == null ? false : .Any(IEE116Repeat =>  == @"R")) &&  != report.SMSFAR1.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430176",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of revocation must be the same as the year of the return",
                    LongDescription = @"The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.",
                    //Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436252" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR1", Value = report.SMSFAR1.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436252

        #region VR.ATO.SMSFAR.436574

        /*  VR.ATO.SMSFAR.436574
                    Net capital gain in return must equal net capital gain in CGT schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([SMSFAR33] <> [CGTS131])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS131] = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    [SMSFAR33] = SMSFAR:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    (CountDocument('CGTS') = 1) AND (^SMSFAR33 <> ^CGTS131)

        Data Elements:
            
                    RP:^SMSFAR33 = tns:Income.CapitalGainsNet.Amount
            
                    RP:^CGTS131 = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    */
        protected void VRATOSMSFAR436574(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436574

            assertion = false; // = (@"CGTS") == 1 && report.SMSFAR33.GetValueOrDefault() != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.SMSFAR.436574",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gain in return must equal net capital gain in CGT schedule",
                    LongDescription = @"Net capital gain amount on the company return  must equal the net capital gain amount on the CGT schedule",
                    //Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436574" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR33", Value = report.SMSFAR33.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436574

        #region VR.ATO.SMSFAR.436801

        /*  VR.ATO.SMSFAR.436801
                    The total of the 'Net capital losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Net capital losses carried forward to later income years' on the main form

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "LS") = 1) AND [LS52] <> [SMSFAR380]  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [LS52] = LS:RP.ALL(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [SMSFAR380] = SMSFAR:RP.Y0Plus.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (CountDocument('LS') = 1) AND ^LS52 <> ^SMSFAR380

        Data Elements:
            
                    RP.Y0Plus.Closing(Instant):^SMSFAR380 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Y0Plus.Closing(Instant):^LS52 = LS:RP.ALL(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    */
        protected void VRATOSMSFAR436801(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436801

            assertion = false; // = (@"LS") == 1 &&  != report.SMSFAR380.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434087",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in Losses schedule",
                    LongDescription = @"The total of the 'Net capital losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Net capital losses carried forward to later income years' on the main form",
                    //Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436801" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR380", Value = report.SMSFAR380.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436801

        #region VR.ATO.SMSFAR.436802

        /*  VR.ATO.SMSFAR.436802
                    The sum of unapplied net capital losses carried forward on the Capital Gains Tax schedule must equal net capital losses carried forward to later income years

                Legacy Rule Format:
                    IF COUNT(SCHEDULE = "CGTS") = 1 AND ([CGTS124] + [CGTS125]) <> [SMSFAR380]  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS124] = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [CGTS125] = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [SMSFAR380] = SMSFAR:RP.Y0Plus.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    CountDocument('CGTS') = 1 AND (^CGTS124 + ^CGTS125) <> ^SMSFAR380

        Data Elements:
            
                    RP.Y0Plus.Closing(Instant):^SMSFAR380 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Y0Plus.Closing(Instant):^CGTS124 = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Y0Plus.Closing(Instant):^CGTS125 = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    */
        protected void VRATOSMSFAR436802(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.SMSFAR.436802

            assertion = false; // = (@"CGTS") == 1 &&  +  != report.SMSFAR380.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.SMSFAR.436620",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unapplied net capital losses carried forward on CGTS must equal net capital losses carried forward",
                    LongDescription = @"The sum of unapplied net capital losses carried forward to later income years on the Capital Gains Tax schedule must equal net capital losses carried forward to later income years on parent return",
                    //Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.436802" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "SMSFAR380", Value = report.SMSFAR380.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.SMSFAR.436802
        #endregion Cross Form Rules - not supported yet
        #region Manual Rules - Rules marked for manual coding

        #region VR.ATO.GEN.432040

        /*  VR.ATO.GEN.432040
                    Only one EFT bank account may be present

                Legacy Rule Format:
                    IF COUNT (TUPLE(financialinstitutionaccount1.xx.xx:FinancialInstitutionAccount)) > 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    NOTE: Common rule set financialinstitutionaccount1.xx.xx:FinancialInstitutionAccount applies to this tuple except VR.ATO.GEN.432040 (due to SMSFAR having two fininstacc tuples).

            Technical Business Rule Format:
                    Count(^FinancialInstitutionAccount) > 1

        Data Elements:
            
                    ^FinancialInstitutionAccount = tns:FinancialInstitutionAccount
                    */
        protected void VRATOGEN432040(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.432040

            assertion = false; // = (Count(report.RpIncomeTaxRefundFinancialInstitutionAccountCollectionCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432040",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Only one EFT bank account may be present",
                    //Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount/tns:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.432040" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 

                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.432040

        #region VR.ATO.GEN.432040

        /*  VR.ATO.GEN.432040
                    Only one EFT bank account may be present

                Legacy Rule Format:
                    IF COUNT (TUPLE(financialinstitutionaccount1.xx.xx:FinancialInstitutionAccount)) > 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    NOTE: Common rule set financialinstitutionaccount1.xx.xx:FinancialInstitutionAccount applies to this tuple except VR.ATO.GEN.432040 (due to SMSFAR having two fininstacc tuples).

            Technical Business Rule Format:
                    Count(^FinancialInstitutionAccount) > 1

        Data Elements:
            
                    ^FinancialInstitutionAccount = tns:FinancialInstitutionAccount
                    */
        protected void VRATOGEN432040_(List<ProcessMessageDocument> response, SMSFAR2017 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.432040

            assertion = false; // = (Count(report.RpIncomeTaxRefundFinancialInstitutionAccountCollectionCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432040",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Only one EFT bank account may be present",
                    //Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount/tns:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.432040" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 

                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.432040
        #endregion Manual Rules - Rules marked for manual coding
        #region ValidateCrossFormRules

        public List<ProcessMessageDocument> ValidateCrossFormRules(SMSFAR2017 report)
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            #region Cross Form Rules
            VRATOGEN438022(response, report);
            VRATOGEN438023(response, report);
            VRATOGEN438024(response, report);
            VRATOGEN438025(response, report);
            VRATOGEN438026(response, report);
            VRATOGEN438027(response, report);
            VRATOSMSFAR436010(response, report);
            VRATOSMSFAR436038(response, report);
            VRATOSMSFAR436051(response, report);
            VRATOSMSFAR436052(response, report);
            VRATOSMSFAR436053(response, report);
            VRATOSMSFAR436072(response, report);
            VRATOSMSFAR436118(response, report);
            VRATOSMSFAR436127(response, report);
            VRATOSMSFAR436129(response, report);
            VRATOSMSFAR436194(response, report);
            VRATOSMSFAR436196(response, report);
            VRATOSMSFAR436197(response, report);
            VRATOSMSFAR436252(response, report);
            VRATOSMSFAR436574(response, report);
            VRATOSMSFAR436801(response, report);
            VRATOSMSFAR436802(response, report);
            #endregion Cross Form Rules
            return response;
        }

        #endregion
    }
}
